from flask import Flask, render_template
import os

###############

import torch
import models
import cv2
import matplotlib as plt
import argparse
import numpy as np
import imutils
import time
from imutils.video import VideoStream

from imutils.video import WebcamVideoStream

# kamera wechsel: für laptop muss(0) extern(1)
#       vs = VideoStream(src=1)
#       cv2.VideoCapture(1) hier sind zwei mal

protoPath = "./face_detector/deploy.prototxt"
modelPath = "./face_detector/res10_300x300_ssd_iter_140000.caffemodel"
print(protoPath)
print(modelPath)
net = cv2.dnn.readNetFromCaffe(protoPath, modelPath)

protoPath2 = "./face_alignment/2_deploy.prototxt"
modelPath2 = "./face_alignment/2_solver_iter_800000.caffemodel"

net2 = cv2.dnn.readNetFromCaffe(protoPath2, modelPath2)


def detector(img):
    frame = imutils.resize(img, width=600)
    (h, w) = frame.shape[:2]
    blob = cv2.dnn.blobFromImage(cv2.resize(frame, (300, 300)), 1.0, (300, 300), (104.0, 177.0, 123.0))
    net.setInput(blob)
    detections = net.forward()

    for i in range(0, detections.shape[2]):
        confidence = detections[0, 0, i, 2]

        if confidence > 0.5:
            box = detections[0, 0, i, 3:7] * np.array([w, h, w, h])
            (startX, startY, endX, endY) = box.astype("int")

            startX = max(0, startX)
            startY = max(0, startY)
            endX = min(w, endX)
            endY = min(h, endY)

            face = frame[startY:endY, startX:endX]
            face = cv2.resize(face, (400, 400))
            return face


def crop_with_ldmk(image, landmark):
    scale = 3.5
    image_size = 224
    ct_x, std_x = landmark[:, 0].mean(), landmark[:, 0].std()
    ct_y, std_y = landmark[:, 1].mean(), landmark[:, 1].std()

    std_x, std_y = scale * std_x, scale * std_y

    src = np.float32([(ct_x, ct_y), (ct_x + std_x, ct_y + std_y), (ct_x + std_x, ct_y)])
    dst = np.float32([((image_size - 1) / 2.0, (image_size - 1) / 2.0),
                      ((image_size - 1), (image_size - 1)),
                      ((image_size - 1), (image_size - 1) / 2.0)])
    retval = cv2.getAffineTransform(src, dst)
    result = cv2.warpAffine(image, retval, (image_size, image_size), flags=cv2.INTER_LINEAR,
                            borderMode=cv2.BORDER_CONSTANT)
    return result


def demo(img):
    data = np.transpose(np.array(img, dtype=np.float32), (2, 0, 1))
    data = data[np.newaxis, :]
    data = torch.FloatTensor(data)
    with torch.no_grad():
        outputs = model(data)
        outputs = torch.softmax(outputs, dim=-1)
        preds = outputs.to('cpu').numpy()
        attack_prob = preds[:, ATTACK]
    return attack_prob


model_name = "MyresNet18"
load_model_path = "a8.pth"
model = getattr(models, model_name)().eval()
model.load(load_model_path)
model.train(False)

ATTACK = 1
GENUINE = 0
thresh = 0.92
testfake = []
#################


from deepface import DeepFace
import cv2
import numpy as np

app = Flask(__name__)


@app.route('/')
def index(name=None):
    return render_template('index.html', name=name)


@app.route('/Verifizierung')
def parse():
    image1 = 'cropped_face/' + os.listdir('cropped_face/')[0]
    print('image1', image1)
    verify1 = ''
    verify2 = ''
    verify3 = ''
    verify4 = ''
    try:
        image2 = 'cropped_face/' + os.listdir('cropped_face/')[1]
        resp = DeepFace.verify(image1, image2, enforce_detection=False)
        print(resp["verified"])
        if resp["verified"] == True:
            verify = 'Gleiche Person'
            print('Gleiche Person')
        else:
            verify = 'Nicht gleiche Person'
            print('Nicht gleiche Person')
        for f in os.listdir('cropped_face/'):
            os.remove(
                os.path.join('cropped_face/', f))
    except IndexError:
        verify = 'Bitte den Vorgang wiederholen! '
        verify1 = 'Achten Sie dabei auf folgende Punkte:'
        verify2 = '1. Positionieren Sie sich nicht zu weit von der Kamera entfernt. '
        verify3 = '2. Es darf nur eine Person im Kamerabild sichtbar sein.'
        verify4 = '3. Halten Sie Ihren Personalausweis neben Ihr Gesicht. Der Personalausweis darf das Gesicht nicht verdecken.'

        for f in os.listdir('cropped_face/'):
            os.remove(
                os.path.join('cropped_face/', f))

    os.remove(os.path.join('faces_detected.jpg'))

    # os._exit(0)

    return render_template('index.html', verify=verify, verify1=verify1, verify2=verify2, verify3=verify3,
                           verify4=verify4)


@app.route('/Gesichtserkennung')
def parse1():
    print('[DEBUG] call cv2.VideoCapture(0) from PID', os.getpid())
    print("[INFO] starting video stream...")
    vs = VideoStream(src=0).start()

    print('vs', vs)

    time.sleep(1.0)

    while True:
        print('hier')

        frame = vs.read()

        # print('frame', frame)
        # frame = vs.read()[1]
        frame = imutils.resize(frame, width=600)

        (h, w) = frame.shape[:2]
        blob = cv2.dnn.blobFromImage(cv2.resize(frame, (300, 300)), 1.0, (300, 300), (104.0, 177.0, 123.0))
        net.setInput(blob)

        detections = net.forward()
        print('detections', detections)

        for i in range(0, detections.shape[2]):

            confidence = detections[0, 0, i, 2]
            print('confidence', confidence)
            if confidence > 0.5:
                box = detections[0, 0, i, 3:7] * np.array([w, h, w, h])
                (startX, startY, endX, endY) = box.astype("int")
                sx = startX
                sy = startY
                ex = endX
                ey = endY

                ww = (endX - startX) // 10
                hh = (endY - startY) // 5

                startX = startX - ww
                startY = startY + hh
                endX = endX + ww
                endY = endY + hh

                startX = max(0, startX)
                startY = max(0, startY)
                endX = min(w, endX)
                endY = min(h, endY)

                x1 = int(startX)
                y1 = int(startY)
                x2 = int(endX)
                y2 = int(endY)

                roi = frame[y1:y2, x1:x2]
                gary_frame = cv2.cvtColor(roi, cv2.COLOR_RGB2GRAY)
                resize_mat = np.float32(gary_frame)
                m = np.zeros((40, 40))
                sd = np.zeros((40, 40))
                mean, std_dev = cv2.meanStdDev(resize_mat, m, sd)
                new_m = mean[0][0]
                new_sd = std_dev[0][0]
                new_frame = (resize_mat - new_m) / (0.000001 + new_sd)
                blob2 = cv2.dnn.blobFromImage(cv2.resize(new_frame, (40, 40)), 1.0, (40, 40), (0, 0, 0))
                net2.setInput(blob2)
                align = net2.forward()

                aligns = []
                alignss = []
                for i in range(0, 68):
                    align1 = []
                    x = align[0][2 * i] * (x2 - x1) + x1
                    y = align[0][2 * i + 1] * (y2 - y1) + y1
                    cv2.circle(frame, (int(x), int(y)), 1, (0, 0, 255), 2)
                    align1.append(int(x))
                    align1.append(int(y))
                    aligns.append(align1)
                cv2.rectangle(frame, (sx, sy), (ex, ey), (0, 0, 255), 2)
                alignss.append(aligns)

                ldmk = np.asarray(alignss, dtype=np.float32)
                ldmk = ldmk[np.argsort(np.std(ldmk[:, :, 1], axis=1))[-1]]
                img = crop_with_ldmk(frame, ldmk)

                attack_prob = demo(img)

                true_prob = 1 - attack_prob
                if attack_prob > thresh:
                    label = 'fake'
                    testfake.append(label)

                    cv2.putText(frame, label + ' :' + str(attack_prob), (sx, sy - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.5,
                                (0, 0, 255), 2)
                else:
                    label = 'true'
                    testfake.append(label)

                    cv2.putText(frame, label + ' :' + str(true_prob), (sx, sy - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.5,
                                (0, 0, 255), 2)

        # print('frame3', frame)
        print('cv2.imshow("Frame", frame)!!!!!!!!!!!!', cv2.imshow("Frame", frame))

        cv2.imshow("Frame", frame)

        key = cv2.waitKey(20) & 0xFF
        if key == ord("q"):
            sfake = 'fake'
            strue = 'true'
            countfake = testfake.count(sfake)
            counttrue = testfake.count(strue)
            print('countfake', countfake)
            print('counttrue', counttrue)
            if (len(testfake) - countfake) < 10:
                verify = "Nicht echte Person"

                vs.stop()
                vs.stream.release()
                cv2.destroyAllWindows()

                break
            else:

                print('ich bin da')
                vs.stop()
                vs.stream.release()
                cv2.destroyAllWindows()

                verify = "Echte Person"
                print('richtige erkennung')
                ########
                ctr = 0
                # import face detection cascade
                face_cascade = cv2.CascadeClassifier('haarcascade_frontalface_default.xml')
                # create capture object
                cap = cv2.VideoCapture(0)

                while True:
                    # capture frame-by-frame
                    ret, img1 = cap.read()
                    # convert image to grayscale
                    gray = cv2.cvtColor(img1, cv2.COLOR_BGR2GRAY)
                    faces = face_cascade.detectMultiScale(img1, 1.3, 5)
                    # for each face draw a rectangle around and copy the face

                    for (x, y, w, h) in faces:
                        cv2.rectangle(img1, (x, y), (x + w, y + h), (255, 0, 0), 2)
                        roi_color = img1[y:y + h, x:x + w]
                        # print("[INFO] Object found. Saving locally.")
                        # cv2.imwrite(str(w) + str(h) + '_faces.jpg', roi_color)
                    # display the resulting frame
                    cv2.imshow('frame', img1)

                    # key = cv2.waitKey(1)
                    key = cv2.waitKey(10) & 0xFF
                    if key == 32:  # if space key is pressed

                        status = cv2.imwrite(
                            'faces_detected.jpg',
                            img1)
                        # print("[INFO] Image faces_detected.jpg written to filesystem: ", status)

                        image = cv2.imread(
                            'faces_detected.jpg')
                        gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

                        faces = face_cascade.detectMultiScale(
                            gray,
                            scaleFactor=1.3,
                            minNeighbors=3,
                            minSize=(30, 30)
                        )

                        print("[INFO] Sind {0} Faces gefunden.".format(len(faces)))

                        for (x, y, w, h) in faces:
                            cv2.rectangle(image, (x, y), (x + w, y + h), (0, 255, 0), 2)
                            roi_color = image[y:y + h, x:x + w]
                            print("[INFO] Object gefunden. Lokal gespeichert.")
                            cv2.imwrite(
                                "cropped_face/face_" + str(
                                    w) + str(h) + ".jpg", roi_color)

                        cap.release()
                        cv2.destroyAllWindows()

                        break

                vs.stop()
                vs.stream.release()
                cap.release()
                cv2.destroyAllWindows()
                break

    # when everything done, release the capture
    cv2.destroyAllWindows()

    return render_template('index.html', verify=verify)


@app.route('/Qrcode')
def parse2():
    cap = cv2.VideoCapture(0)
    # initialize the cv2 QRCode detector

    detector = cv2.QRCodeDetector()

    while True:
        success, img = cap.read()
        # detect and decode
        data, bbox, _ = detector.detectAndDecode(img)
        # check if there is a QRCode in the image
        if data:
            verify = data
            print(verify)
            break
        if success:
            print('sucesss!!!')
            cv2.imshow("QRCODEscanner", img)

        if cv2.waitKey(1) == ord("q"):
            break

    cap.release()
    cv2.destroyAllWindows()

    return render_template('index.html', verify=verify)


if __name__ == '__main__':
    app.run(debug=True)
